$LOAD_PATH.unshift __dir__

require "tasky/version"
require "optparse"
require "fileutils"

module Todo
  class Options
    def self.parse(args)
      options = {}

      opt_parser = OptionParser.new do |opts|
        opts.banner = "Usage: example.rb [options]"

        opts.on("-l", "--list", "Print todo tasks") do
          Todo.print_tasks
        end

        opts.on("-d", "--done", "Print done tasks") do
          Todo.print_tasks(:done)
        end

        opts.on("-e", "--edit", "Edit tasks file") do
          Todo.edit
        end

        opts.on_tail("-h", "--help", "Prints this help") do
          puts opts
          exit
        end

        opts.on_tail("-v", "--version", "Prints version") do
          puts "todo #{VERSION}"
          exit
        end
      end

      opt_parser.parse!(args)
      options
    end
  end

  class << self
    def start(args)
      options = Options.parse args

      @task = Task.new args

      @task.save
    end

    def print_tasks(status = :todo)
      pattern = status == :todo ? /^\[x/ : /^\[ /
      File.open(Task.file).each_line do |l|
        puts(l) unless l[pattern]
      end
      exit
    end

    def edit
      exec("#{editor} #{Todo::Task.file}")
    end

    def editor
      ENV['VISUAL'] || ENV['EDITOR'] || 'vim'
    end

    def help
      puts 'Usage: todo [options] <args>'
      exit
    end

    def version
      VERSION
    end
  end

  class Task
    class InvalidStatus < StandardError; end

    attr_accessor :message

    def initialize(message)
      Todo.help if message.empty?

      @message = '[ ] ' + message.join(' ')

      # make sure the file exists
      FileUtils.touch(file)
    end

    def save
      File.open(file, 'a') do |f|
        f.puts message
      end

      File.size(file)
    end

    def size
      message.size
    end

    def file
      File.expand_path(File.join('~', '.todo'))
    end

    # wrapper to Task#file with a dummy task
    def self.file
      new([?.]).file
    end
  end
end
